package body algoritmos_Grafos is
	Type Marcas is array(Positive Range <>) of Boolean;
	-- Recorre un grafo en profundidad.
	procedure Profundidad(G: in Grafo'Class) is
		-- Recorre en profundidad los vrtices de G accesibles
		-- desde V, Tratndolos y marcndolos como
		-- visitados.
		procedure Avanzar_En_Profundidad
					(G: in Grafo'Class;
					 V: in Positive;
					 Visitados: in out Marcas) is
		begin
			-- El vrtice V se marca como visitado	
			Visitados(V) := True;

			-- Se trata el vrtice V.
			Tratar(V);

			-- Se examinan los vrtices adyacentes a V para
			-- continuar el recorrido.
			for i in 1 .. Vrtices(G) loop
				if (V /= I) 
				   and then not(Visitados(I))
				   and then Conectados(G,V,I) then
					Avanzar_En_Profundidad(G,I,Visitados);
				end if;
			end loop;
		end Avanzar_En_Profundidad;

		-- Visitados se pone a True a medida que se visitan los
		-- vrtices.
		Visitados: Marcas(1..Vrtices(G)) := (others => false);
	begin
		-- El recorrido se relanza en cada vrtice no visitado
		-- hasta que lo estn todos.
		for i in 1 .. Vrtices(G) loop
			if not Visitados(i) then
				Avanzar_En_Profundidad(G, I, Visitados);
			end if;
		end loop;
	end Profundidad;

	procedure Amplitud(G: in Grafo'Class) is
		-- Se declara una cola capaz de mantener todos los vrtices de G
		En_Curso: Cola(Vrtices(G));

		-- Visitados se pone a True a medida que se visitan los
		-- vrtices.
		Visitados: Marcas(1..Vrtices(G)) := (others => false);

		V: Natural; -- V representa el vrtice actual
	begin
		-- El recorrido en amplitud se inicia en cada vrtice
	 	-- an no visitado.
		for i in 1 .. Vrtices(G) loop
			if not Visitados(I) then
				-- Se pone en la cola el vrtice de partida
				-- y se marca como visitado.
				Insertar(En_curso,I);
				Visitados(I) := True;

				while not Es_Vaca(En_Curso) loop
					-- Se toma el vrtice que est en el frente
					-- de la cola y se trata.
					V := Examinar(En_Curso);
					Extraer(En_Curso);
					Tratar(V);

					-- Todos los vrtices adyacentes a V no visitados
					-- se incorporran a la cola y se marcan.
					for j in 1.. Vrtices(G) loop
						if (V /= J) and then Conectados(G,V,J) then
							if (not Visitados(J)) then
								Insertar(En_Curso,j);
								Visitados(J) := true;
							end if;
						end if;
					end loop;
				end loop;
			end if;
		end loop;
	end Amplitud;

	-- Costes_Desde_Fuente calcula el coste y vrtice responsable de cada
	-- camino de coste mnimo deste Fuente a cada vrtice de G
	procedure Costes_Desde_Fuente(G: in Grafo_Costes; Fuente: In Positive;
						Caminos: out Costes_Mnimos) is

		Coste_Min,				-- Coste mnimo en una iteracin
		Coste_Aux  : natural;	-- Variable para recalcular costes
		V_Coste_Min: Natural;	-- Vrtice que consigue el coste mnimo
								-- en una iteracin
		Tratados   : marcas(1..Vrtices(G)) := (others => False);
							 	-- Vrices ya tratados
	begin
		-- Inicializar el conjunto de vrtices tratados con la fuente y
	 	-- calcular el coste directo desde la fuente a cada vrtice
		Tratados(Fuente) := True;

		for i in 1 .. Vrtices(G) loop
			Caminos(i).coste := Coste(G,Fuente,I);
			if Conectados(G,Fuente,I) then
				Caminos(I).Vrtice := Fuente;
			else
				Caminos(i).Vrtice := 0;
			end if;
		end loop;
		-- A la fuente se le asigna con coste directo cero 
		Caminos(Fuente).Vrtice:= Fuente;
		Caminos(Fuente).Coste := 0;

		-- Se calculan los costes mnimos desde Fuente a 
		-- todos los dems vrtices
		for j in 1 .. Vrtices(G) - 1 loop
			-- Se determina el nuevo vrtice pendiente de coste
		 	-- mnimo desde Fuente
			Coste_Min  := Natural'Last;
			V_Coste_min:= 0;
			for i in 1 .. Vrtices(G) loop
				if not Tratados(i) then
					if Caminos(i).Coste <= Coste_Min then
						Coste_Min   := Caminos(i).Coste;
						V_Coste_Min := i;
					end if;
				end if;
			end loop;

			-- Se aade el nuevo vrtice de coste mnimo al conjunto de
		 	-- los tratados y se actualizan los costes de los caminos en
			-- funcin del nuevo vrtice
			Tratados(V_Coste_Min) := True;
			for i in 1 .. vrtices(G) loop
				if not Tratados(i) then
					begin
						Coste_Aux := Coste_Min + Coste(G,V_Coste_Min,i);
					exception  -- Por si la suma excede el rango de los naturales
						when Constraint_Error => Coste_Aux := Natural'Last;
					end;
					if Caminos(i).coste > Coste_Aux then
						Caminos(i).Coste   := Coste_Aux;
						Caminos(i).Vrtice := V_Coste_Min;
					end if;
				end if;
			end loop;
		end loop;
	end Costes_Desde_Fuente;

	-- Devuelve un conjunto de aristas que forma un rbol de recubrimiento
	-- de coste mnimo del grafo G
	procedure rbol_De_recubrimiento(G: in Grafo_Costes;
						Aristas: out Conjunto_Aristas) is
		Costes_Bajos : Costes_Mnimos(1..Vrtices(G));
		n_Aristas    : Natural := 0;
		Mnimo, VMin : Natural;
		Conectados   : marcas(1..Vrtices(G)) := (others => False);
	begin
		-- Se toma el vrtice 1 como primer vrtice del rbol de 
		-- recubrimiento y se calculan las aristas de coste mnimo
		-- desde el vrtice 1 a cada uno de los otros
		for i in 1 .. Vrtices(G) loop
			costes_Bajos(i).Coste := Coste(G,1,i);
			Costes_Bajos(I).Vrtice := 1;
		end loop;
		Conectados(1) := True;
		
		-- Se completa el rbol de recubrimiento seleccionando en
		-- cada iteracin la arista de coste mnimo entre los vrtices
		-- conectados y los no conectados.
		for i in 2 .. Vrtices(G) loop
			-- Se busca la arista de coste mnimo
			Mnimo := Natural'Last;
			VMin := 0;
			for j in 2 .. Vrtices(G) loop
				if (not Conectados(j)) and then
					Costes_Bajos(j).Coste < Mnimo then
					Mnimo := Costes_Bajos(j).Coste;
					VMin := j;
				end if;
			end loop;

			-- Se incluye la arista seleccionada en el rbol de
			-- recubrimiento de coste mnimo
			N_Aristas := N_Aristas + 1;
			Aristas(N_Aristas).V1    := VMin;
			Aristas(N_Aristas).V2    := Costes_Bajos(VMin).Vrtice;
			Aristas(N_aristas).Coste := Costes_Bajos(VMin).Coste;
			Conectados(VMin)        := True;

			-- Se recalculan las aristas de coste mnimo entre los vrtices
			-- conectados y los no conectados teniendo en cuenta el vrtice
			-- recin conectado
			for j in 2 .. Vrtices(G) loop
				if (not Conectados(j)) and then
					(Coste(G,VMin,j) < Costes_Bajos(j).Coste) then
					Costes_Bajos(j).coste   := Coste(G,VMin,j);
					Costes_Bajos(j).Vrtice := VMin;
				end if;
			end loop;
		end loop;
	end rbol_De_recubrimiento;

end algoritmos_grafos;
