package Grafos_Simples.Costes is
	TYPE Grafo_Costes(Clase: Clase_Grafo; N: Positive) is 
		new Grafo with private;
	-- Representa grafos de costes, orientados o no orientados, con 
	-- N vrtices nmerados de 1 a N.

	procedure Conectar(G: in out Grafo_Costes;
					   I, J: in Positive;
					   Coste: Natural);
	-- Establece un arco o arista con valor asociado, Coste, entre el
 	-- vrtice I y el J.
	-- Si el grafo es orientado I es el extremo inicial del arco. Si ya 
	-- exista la conexin, se actualiza su coste.


	function Coste(G: in Grafo_Costes;
				   I, J: in Positive) return Natural; 
	-- Devuelve el coste del arco o arista (I,J), si los vtices I y J
	-- estn conectados. Si no lo estn, devuelve Natural'Last.

private
	-- Matriz para representar los costes.
	type Matriz_Costes is
		array(Positive range <>,Positive range <>)
		of natural;

	-- El grafo se crea con N vrtices desconectados. La matriz de
	-- costes se rrellena inicialmente con el mayor coste posible
	-- (Natural'Last);
	type Grafo_Costes(Clase: Clase_Grafo; N: Positive) is 
			new Grafo(Clase,N) with record
		Costes: Matriz_Costes(1..N,1..N) := 
			(Others => (others => Natural'Last));
	end record;
		
end Grafos_simples.Costes;
