#include "grafocostes.h"
#include <queue>

namespace algoritmosGrafos {
   // Estructura que representa un vrtice y un coste relacionado con l
   struct CosteMin {
      int vertice, coste;
      CosteMin() {}
      CosteMin(int v, int c):vertice(v),coste(c) {}
   };

   // Estructura que representa una arista y su coste
   struct Arista {
      int v1, v2, coste;
      Arista() {}
      Arista(int a, int b, int c):v1(a),v2(b),coste(c) {}
   };

   /**
    * Devuelve un conjunto de aristas que forma un rbol de recubrimiento
    * de coste mnimo del grafo g
    * @return un array con las aristas del rbol de recubrimiento de coste mnimo
    */
   Arista *arbolDeRecubrimiento(GrafoCostes g);

   /**
    * Calcula el coste y vrtice responsable de cada camino
    * de coste mnimo deste fuente a cada vrtice de g
    * @return un array con los costes de los caminos de la fuente a cada vrtice
    */	
   CosteMin *costesDesdeFuente(GrafoCostes g, int fuente);

   /**
    * Recorre un grafo en amplitud
    */ 		
   void amplitud(Grafo g, void (*tratar)(int));

   /**
    * Recorre en profundidad los vrtices de g accesibles desde v,
    * tratndolos y marcndolos como visitados.
    */ 	
   void avanzarEnProfundidad(Grafo &g,int v,bool *visitados, void (*tratar)(int));

   /**
    * Recorre un grafo en profundidad
    */	
   void profundidad(Grafo &g, void (*tratar)(int));
}


