#ifndef INC_GRAFOS
#define INC_GRAFOS

typedef enum {ORIENTADO,NoORIENTADO} ClaseGrafo;

class Grafo {
   public:
      /**
       * Constructor: crea la matriz de adyacencia que representa al grafo
       * y lo inicializa adecuadamente
       */
      Grafo(ClaseGrafo claseGrafo, int n) {
         clase = claseGrafo;
         nV    = n;
         adyacencia = new bool*[n];
         for (int i = 0; i < nV; i++){
            adyacencia[i] = new bool[n];
            for (int j = 0; j < nV; j++)
               adyacencia[i][j] = false;
         }
      }
   
      /**
       * Constructor de copia
       */
      Grafo(const Grafo &g) {
         clase = g.clase;
         nV    = g.nV;
         adyacencia = new bool*[nV];
         for (int i = 0; i < nV; i++){
            adyacencia[i] = new bool[nV];
            for (int j = 0; j < nV; j++)
               adyacencia[i][j] = g.adyacencia[i][j];
         }
      }

      /**
       * Destructor: libera la memoria ocupada por la matriz de adyacencia
       */
      virtual ~Grafo() {
         for (int i = 0; i < nV; i++)
            delete adyacencia[i];
         delete adyacencia;
      }
   
      /**
       * Establece un arco o arista entre el vrtice i y el j.
       * Si el grafo es orientado i es el extremo inicial del arco.
       */
      void conectar(int i, int j) {
         adyacencia[i][j] = true;
         if (clase == NoORIENTADO) adyacencia[j][i] = true;
      }
   
      /**
       * Elimina el arco o arista que haya entre el vrice i y el j.
       * Si el grafo es orientado i es el extremo inicial del arco.
       * Si el arco o arista a eliminar no existe no ocurre nada.
       */
      void desconectar(int i, int j) {
         adyacencia[i][j] = false;
         if (clase == NoORIENTADO) adyacencia[j][i] = false;
      }
   
      /**
       * Devuelve true si existe un arco o arista entre el vrtice
       * i y el j, y false si no.
       * Si el grafo es orientado i es el extremo inicial del arco.
       */
      bool conectados(int i, int j) {
         return adyacencia[i][j];
      }
   
      /**
       * Devuelve el nmero de vrtices del grafo.
       */
      int vertices() {
         return nV;
      }

      /**
       * Operador de asignacin
       */
      Grafo & operator =(const Grafo &g) {
         if (this != &g) // Si no es una autoasignacin
         {
            for (int i = 0; i < nV; i++)
               delete adyacencia[i];
            delete adyacencia;

            clase = g.clase;
            nV    = g.nV;
            adyacencia = new bool*[nV];
            for (int i = 0; i < nV; i++){
               adyacencia[i] = new bool[nV];
               for (int j = 0; j < nV; j++)
                  adyacencia[i][j] = g.adyacencia[i][j];
            }
         }
         return *this;
      };
   
   protected:
      ClaseGrafo clase;  // ORIENTADO/NoORIENTADO
      int nV;            // nmero de vrtices
      bool **adyacencia; // matriz de adyacencia
   
};

#endif
