#ifndef INC_GRAFO_COSTES
#define INC_GRAFO_COSTES
#include <climits>
#include "grafo.h"
/**
 * Implementacin de un grafo simple de costes formado por vrtices que  
 * son nmeros naturales y arcos que los conectan con coste positivo.
 */
class GrafoCostes:public Grafo {
   public:
      /**
       * Constructor: despus de invocarse al constructor de la clase base
       * se crea la matriz de costes y se inicializa adecuadamente
       */   
      GrafoCostes(ClaseGrafo claseGrafo, int n):Grafo(claseGrafo,n) {
         costes = new int*[n];
         for (int i = 0; i < vertices(); i++){
            costes[i] = new int[n];
            for (int j = 0; j < vertices(); j++)
               costes[i][j] = INT_MAX;
         }
      }
   
      /**
       * Constructor de copia
       */
      GrafoCostes(const GrafoCostes &g):Grafo(g) {
         costes = new int*[nV];
         for (int i = 0; i < nV; i++){
            costes[i] = new int[nV];
            for (int j = 0; j < nV; j++)
               costes[i][j] = g.costes[i][j];
         }
      }


       /**
        * Destructor: libera la memoria ocupada por la matriz de costes
        */
      ~GrafoCostes() {
         for (int i = 0; i < nV; i++)
            delete costes[i];
         delete costes;
      }
   
   
      /**
       * Establece un arco o arista de coste coste entre el vrtice i y el j.
       * Si el grafo es orientado i es el extremo inicial del arco. Si ya 
       * exista la conexin, se actualiza su coste.
       */
      void conectar(int i, int j, int coste) {
         Grafo::conectar(i,j);
         costes[i][j] = coste;
         if (clase == NoORIENTADO) costes[j][i] = coste;
      }
   
      /**
       * Devuelve el coste del arco o arista (I,J), si los vtices I y J
       * estn conectados. Si no lo estn, devuelve el mayor entero.
       */
      int coste(int i, int j) {
         if (conectados(i,j))
            return costes[i][j];
         else
            return INT_MAX; // "infinito"
      }
   
      /**
       * Operador de asignacin
       */
      GrafoCostes & operator =(const GrafoCostes &g) {
         if (this != &g){ // Si no es una autoasignacin
            Grafo::operator=(g);
            for (int i = 0; i < nV; i++)
               delete costes[i];
            delete costes;

            clase = g.clase;
            nV    = g.nV;
            costes = new int*[nV];
            for (int i = 0; i < nV; i++){
               costes[i] = new int[nV];
               for (int j = 0; j < nV; j++)
                  costes[i][j] = g.costes[i][j];
            }
         }
         return *this;
      };


   private:
      int **costes; // Matriz para representar los costes.
};

#endif
