package Grafos;

package grafos;
/**
 * Implementacin de un grafo simple formado por vrtices que son nmeros
 * enteros positivos y arcos que los conectan.
 */

public class Grafo {
	ClaseGrafo clase;	// ORIENTADO/NoORIENTADO
	int nV;				// nmero de vrtices
	boolean adyacencia[][]; // matriz de adyacencia

	public Grafo(ClaseGrafo claseGrafo, int n) {
		clase = claseGrafo;
		nV    = n;
		adyacencia = new boolean[n][n];
		for (int i = 0; i < nV; i++){
			for (int j = 0; j < nV; j++)
				adyacencia[i][j] = false;
		}
	}

	/**
	 * Establece un arco o arista entre el vrtice i y el j.
	 * Si el grafo es orientado i es el extremo inicial del arco.
	 */
	public void conectar(int i, int j) {
		adyacencia[i][j] = true;
		if (clase == ClaseGrafo.NoORIENTADO) adyacencia[j][i] = true;
	}
	
	/**
	 * Elimina el arco o arista que haya entre el vrice i y el j.
	 * Si el grafo es orientado i es el extremo inicial del arco.
	 * Si el arco o arista a eliminar no existen no ocurre nada.
	 */
	public void desconectar(int i, int j) {
		adyacencia[i][j] = false;
		if (clase == ClaseGrafo.NoORIENTADO) adyacencia[j][i] = false;
	}
	
	/**
	 * Devuelve true si existe un arco o arista entre el vrtice
 	 * i y el j, y fase si no.
	 * Si el grafo es orientado i es el extremo inicial del arco.
	 */
	public boolean conectados(int i, int j) {
		return adyacencia[i][j];
	}
	
	/**
	 * Devuelve el nmero de vrtices del grafo.
	 * @return
	 */
	public int vrtices() {
		return nV;
	}
}
